#include <stdio.h>
#include <stdlib.h> 
#include <time.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <iostream>
#include <fstream>
#include <string.h>
#include <algorithm>
#include <vector>
#include <set>
#include <map>
#include "model.h"
#include "anchap.h"
#include "fpmath.h"
#include "control.h"
#include "gsl/gsl_linalg.h"
#include "gsl/gsl_permutation.h"
#include "gsl/gsl_permute.h"
#include "gsl/gsl_cdf.h"
#include "gsl/gsl_statistics.h"
extern "C"{
#include "gsl/gsl_randist.h"
}
#include "gsl/gsl_rng.h"
#include "gsl/gsl_eigen.h"
#include "stats.h"

using namespace std;

double global_Ne; 

void ModelnData::print_progress_bar(int last, char* str, int p, int total)
{
	if(m_silence == 1) return; 
	int progress = (int) (100.0 * p / total); 
	int barsize = (int) (progress / 2.0); 
	char bar[100];
	memset(bar, '\0', 100); 
	if(!last) {
		for (int i = 0; i < barsize; i++)
			bar[i] = '>'; 
		printf("%s [%-50s] %d%%\r", str, bar, progress); 
		fflush(stdout); 	
	} else {
		for (int i = 0; i < barsize; i++)
			bar[i] = '>'; 
		printf("%s [%-50s] 100%%\n", str, bar); 
	}
}

ModelnData::ModelnData(void)
{
	morgan = NULL; 
	snpmgt = NULL; 
	snpIn01 = NULL; 
	m_silence = 0; 
	m_exclude_maf = -0.0; 
	m_exclude_miss = 1.00; 
	m_exclude_nopos = 1; 
	m_exclude_miss1 = 0; 
	m_num = 1;      //this is beta distribution to link between top and bottom layer. 
	m_nph = 4; 
	nK = 10; 
	nS = 2; 
	randSeed = 0;
	nEMRuns = 1;
	nMaxSteps = 0;
	nWarmSteps = 30; 

	global_Ne = 1; 
	nLoci = 0; 
	
	fnOutput.assign("\0");
	fnRsPos.assign("\0");
	
	nGeneFlank = 0; 
	nDip = 0;
	nHap = 0;
	pMP = NULL;

	m_npath = 2; 

	m_upper_switches = 1; 
	m_rrt = 1; 
	m_morgan = -1; 
	m_mixgen = 100; 

	m_verbose = 1; 
	m_impute = 0; 
	m_init = 2; 

	m_mu = 0.0;
	m_cset.clear(); 
	m_xset.clear(); 
}

ModelnData::~ModelnData()
{
	if(pMP) {delete[] pMP;  pMP = NULL;} 
	vGin.clear();
	vPin.clear(); 
	vFileIndiv.clear(); 
	vFilePloid.clear(); 
	vsRsnum.clear(); 
	nLoci = 0; 
	
	nDip = 0;
	nHap = 0;
}

void ModelnData::InitModelParam(void)
{
	if (pMP == NULL)
		pMP = new ModelParam[nEMRuns];
	
	//li and stephens used 4Nc = 20, 200. 
	//the unit is centi-morgan per mega-basepair. 
	//we assume 1 centi-morgan per mega-basepair.
	//combine together we have 50/nK cross-over events per mega-basepair; 
	//then divide by nK because the branch length is 1/nK. 
    //m_morgan has unit 100 Mpb, so we have 5000 / nK. 

	m_upper_switches = m_morgan * m_mixgen; 
	m_lower_switches = m_morgan * 1000.0; 
	double rhok = m_lower_switches / nLoci;  //default can be thought as mixing 1000 generations agao. 
	double rhos = m_upper_switches / nLoci; 
	double rhou = m_rrt / nLoci; 
	fplog << " ### estimated total genetic distance " << m_morgan << endl; 
	fplog << " ### constrained upper layer switches " << m_upper_switches << endl; 
	fplog << " ### constrained lower layer switches " << m_lower_switches << endl; 
	fplog << " ### constrained ancillary switches " << rhou * nLoci << endl; 

	if(rhok > 0.5) rhok = 0.5; 
	if(rhos > 0.5) rhos = 0.5; 
	if(rhou > 0.5) rhou = 0.5; 
	fplog << " ### rhok used for intialization"  << rhok <<  endl; 
	fplog << " ### rhos used for intialization"  << rhos <<  endl; 
	fplog << " ### rhou used for intialization"  << rhou <<  endl; 
    for (int i = 0; i < nEMRuns; i++)
		pMP[i].Init(nLoci,nS, nK, rhok, rhos, rhou, m_init, NULL);
}



void ModelnData::assign_pop_label(void)
{               
    for (int i = 0; i < nIndiv; i++)
	{
		int ph = m_phval.at(i); 
		if(ph < 10) continue; 
	
		if(ph - 10 >= nS) {
			fplog << " ### -p <ph>, where ph < 10+nS "; 
			exit(0); 
		}
	   	pIndiv[i]->set_admix(ph-10, nS); 
	}
}

#include <gsl/gsl_errno.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_roots.h>

struct params
{
	double b; 
	double Ne; 
};

double digamma(double x)
{
	double x1=1.0/x; 
	double x2=x1/x; 
	double x3=x2/x;
	double x4=x3/x;
	double x5=x4/x;

   	double res = x1 + 0.5*x2 + 5.0/24.0 * x3 + 1.0/16.0 * x4 + 47.0 / 48.0 / 120.0 * x5; 
	return (-log(res)); 
}

double f (double x, void * par) 
{               
	struct params * p = (struct params * ) par; 
	double b =  p->b; 
	double Ne =  p->Ne; 
	double nx = Ne * x; 
	double res = digamma(nx + 6.0) - digamma(6.0 + Ne - nx); 
	res -= 1.0 / nx + 1.0 / (nx + 1.0) + 1.0 /(nx + 2.0) + 1.0 / (nx + 3.0) + 1.0 / (nx + 4.0) + 1.0 / (nx + 5.0); 
	res += 1.0 / (Ne - nx) + 1.0 / (Ne - nx + 1.0) + 1.0 / (Ne - nx + 2.0) + 1.0 / (Ne - nx + 3.0) + 1.0 / (Ne -nx + 4.0) + 1.0 / (Ne - nx + 5.0); 
	res -= b; 
	return(res); 
}

void ModelnData::EM_joint(int comp_mode)
{
	int totalsteps = 0; 
	if(comp_mode == 1) totalsteps = nWarmSteps; 
	else if(comp_mode == 2) totalsteps = nMaxSteps; 
	if(totalsteps == 0) return; 
	//comp_mode == 1; iterative marginalization;
	//comp_mode == 2; quadratic;

	int nsk = nS * nK; 

	double ** sum_jmsk = Allocate2DMatrix(nLoci, nsk); 
	double ** sum_jms = Allocate2DMatrix(nLoci, nS); 
	double ** sum_top = Allocate2DMatrix(nLoci, nK); 
	double ** sum_bot = Allocate2DMatrix(nLoci, nK); 

	double ** sum_lod = Allocate2DMatrix(nLoci, nS); 
	double ** sum_lps = Allocate2DMatrix(nLoci, nS); 

	double * sum_ujs = new double[nLoci]; 

	for (int i = 0; i < nIndiv; i++)
		pIndiv[i]->clean_after_em(1); 

	for (int e = 0; e < nEMRuns; e++)
	{
		double lastlike = 0; 

  		assign_pop_label(); 
		for (int steps = 0; steps < totalsteps; steps++)
		{
			double totalloglike = 0; 
			
			for (int m = 0; m < nLoci; m++)              
			{
				sum_ujs[m] = 0.0; 
				for (int j = 0; j < nsk; j++)
					sum_jmsk[m][j] = 0.01; 
				for (int s = 0; s < nS; s++)
					sum_jms[m][s] = 0; 
				for (int j = 0; j < nK; j++)
					sum_top[m][j] = sum_bot[m][j] = 0.0; 
				for (int s = 0; s < nS; s++)
					sum_lod[m][s] = sum_lps[m][s] = 0; 
			}

			///////////////////////////////////////////////
			double ** th = (pMP+e)->Gettheta(); 
			for (int i = 0; i < nK; i++)
			{
				for (int m = 0; m < nLoci; m++)
					pHap[i]->set_af(m, th[m][i]); 
				pHap[i]->CalcAll(nLoci, nS, pMP+e, sum_top, sum_bot, sum_lod, sum_lps, i, 1); 
				double * js = pHap[i]->get_pjs(); 
				for (int m = 0; m < nLoci; m++)
					sum_ujs[m] += js[m]; 
				pHap[i]->clean_after_em(1); 
			}
			/////////////////////////////////////////////

			for (int m = 1; m < nLoci; m++)
			{
				double tru = sum_ujs[m] / nK;
				if(tru > 1) tru = 1; 
				(pMP+e)->Setru(m, tru); 
			}
			//update eta; 
			/////////////////////////////////////////////////////////////////////
			if(fabs(global_Ne - 1) > 1e-6) 
			{
				int iter = 0, max_iter = 100;
				const gsl_root_fsolver_type *T;
				gsl_root_fsolver *solver;
				gsl_function FDF;
				struct params para;
				para.Ne = global_Ne; 

				FDF.function = &f;
	//			FDF.df = &df;
	//			FDF.fdf = &fdf;
				FDF.params = &para;

				T = gsl_root_fsolver_brent;
				solver = gsl_root_fsolver_alloc (T);

				for (int m = 0; m < nLoci; m++)
				{
					for (int s = 0; s < nS; s++)
					{
						double x = 0.5;
						double ss = sum_lod[m][s] / sum_lps[m][s]; 
						if(ss > 1e10) 
							x = 0.999; 
						else 
						{
							double x_lo = 0.001, x_hi = 0.999; 
							para.b = ss; 
							int status;
							iter = 0;
							gsl_root_fsolver_set (solver, &FDF, x_lo, x_hi);
							do {
								iter++;
								status = gsl_root_fsolver_iterate (solver);
								x = gsl_root_fsolver_root (solver);
								x_lo = gsl_root_fsolver_x_lower (solver);
								x_hi = gsl_root_fsolver_x_upper (solver);
								status = gsl_root_test_interval (x_lo, x_hi, 0, 0.001);
							 }
							while (status == GSL_CONTINUE && iter < max_iter);
						}
						(pMP+e)->Seteta(m, s, x); 
					}   //eta; 
				}
				gsl_root_fsolver_free (solver);
			}
			else
			{
				for (int m = 0; m < nLoci; m++)
					for (int s = 0; s < nS; s++)
					{
						double tt = 0.5; 
						double lod = sum_lod[m][s] / sum_lps[m][s]; 
						if(fabs(lod) > 1e-20) 
						{
							lod /= -3.1415926;  
							tt = 0.5 - atan(lod) / 3.1415925;   
							if(tt > 0.999) tt = 0.999; 
							else if(tt < 0.001) tt = 0.001; 
						}
						(pMP+e)->Seteta(m, s, tt); 
					}   //eta; 
			}
			///////////////////////////////////////////////////////////

			assign_pop_label();                          
			double ploids = 0; 
		    fplog << "### "; 
			for (int i = 0; i < nIndiv; i++)
			{
				if(pIndiv[i]->get_mask() == 1) continue;
				if(comp_mode == 1)  //-w
				{
					pIndiv[i]->CalcAll(nS, nK, pMP+e, sum_top, sum_bot, sum_jmsk, sum_jms, 1, 0); 
					ploids += pIndiv[i]->Getploid(); 
				}
				else          //-s 
				{
					pIndiv[i]->CalcAll(nS, nK, pMP+e, sum_top, sum_bot, sum_jmsk, sum_jms, 2, 0); 
					ploids += pIndiv[i]->Getploid(); 
				}
				totalloglike += pIndiv[i]->GetLikelihood(); 
				pIndiv[i]->clean_after_em(1); 
			}

			//update theta; 
//			for (int m = 0; m < nLoci; m++)
//			{
//				for (int k = 0; k < nK; k++)
//				{                      
//					double tt = 0.5; 
//					double lod = sum_top[m][k] / sum_bot[m][k]; 
//					if(fabs(lod) > 1e-20) 
//					{
//						lod /= -3.1415926;  
//						tt = 0.5 - atan(lod) / 3.1415925;   
//						if(tt > 0.999) tt = 0.999; 
//						else if(tt < 0.001) tt = 0.001; 
//					}
//					//bound theta; 
//					if(isnan(tt)) {
//						cout << "tt is nan in theta unpdate " << endl; 
//				   		(pMP+e)->Settheta(m, k, 0.5); 
//					}
//					else
//				   		(pMP+e)->Settheta(m, k, tt); 
//				}   //share the lower clusters; 
//			}
			for (int m = 0; m < nLoci; m++)
			{
				for (int k = 0; k < nK; k++)
				{                      
					double tt = (sum_top[m][k] + 1e-20) / (sum_bot[m][k] + 2e-20); 
					if(tt > 0.999) tt = 0.999; 
					else if(tt < 0.001) tt = 0.001; 
					//bound theta; 
					if(isnan(tt)) {
						fplog << "tt is nan in theta unpdate " << m << " " << sum_top[m][k] << " " << sum_bot[m][k] << endl; 
						cout << "tt is nan in theta unpdate " << m << " " << sum_top[m][k] << " " << sum_bot[m][k] << endl; 
//						exit(0); 
				   		(pMP+e)->Settheta(m, k, 0.5); 
					}
					else
				   		(pMP+e)->Settheta(m, k, tt); 
				}   //share the lower clusters; 
			}

			//update beta; 
			for (int m = 0; m < nLoci; m++)
	   		{                 
				double jk = 0; 
				for (int s = 0; s < nS; s++)
				{
					double total = 0; 
					for (int j = s * nK, k = 0; k < nK; k++, j++)
						total += sum_jmsk[m][j]; 
					jk += total; 
					if(total < 1e-100) { cout << "total tiny " << m << " " << s << endl; continue;} 
					for (int j = s * nK, k = 0; k < nK; k++, j++)
					{
						double ta = sum_jmsk[m][j] / total;
						if(isnan(ta)) { 
                            fplog << "ta is nan in beta update " << endl; 
                            cout << "ta is nan in beta update " << endl; 
							(pMP+e)->Setbeta(m, s, k, 1.0 / nK);
						}
						else
							(pMP+e)->Setbeta(m, s, k, ta);
					}
				}
				double trk = jk / ploids; //coupling; 
				if(trk > 1) trk = 1; 
//				if(isnan(trk)) {
//					fplog << "trk is nan in rk upate " << endl; 
//					cout << "trk is nan in rk upate " << endl; 
//					(pMP+e)->Setrk(m, 1e-20);
//				}
//				else 
//					(pMP+e)->Setrk(m, trk); 
				//set rk; 

				double js = 0; 
				for (int s = 0; s < nS; s++)
					js += sum_jms[m][s]; 
				double trs = js / ploids; //coupling; 
				if(trs > 1) trs = 1; 
				if(isnan(trs)) {
					fplog << "trs is nan in rs upate " << endl; 
					cout << "trs is nan in rs upate " << endl; 
					(pMP+e)->Setrs(m, 1e-20);
				}
				else 
					(pMP+e)->Setrs(m, trs); 
				//set rs; 
			}
			(pMP+e)->Setrs(0, 1); 
			(pMP+e)->Setrk(0, 0); 
			
			double ktotal = 0; 
			double * rk = (pMP+e)->Getrk(); 
			for (int m = 1; m < nLoci; m++)
   				ktotal += rk[m]; 
//			if(ktotal > m_lower_switches) 
//			{
//				for (int m = 1; m < nLoci; m++)
//				{
//					double tr = rk[m]; 
//					(pMP+e)->Setrk(m, 1-exp(log(1-tr) / ktotal * m_lower_switches)); 
//				}
//			}
			
			double utotal = 0; 
			double * ru = (pMP+e)->Getru();
			for (int m = 1; m < nLoci; m++)
				utotal += ru[m]; 

//			if(utotal > m_rrt) 
//			{
//				for (int m = 1; m < nLoci; m++)
//				{
//					double tr = ru[m]; 
//					(pMP+e)->Setru(m, 1-exp(log(1-tr) / utotal * m_rrt)); 
//				}
//			}

			double stotal = 0; 
			double * rs = (pMP+e)->Getrs();
			for (int m = 1; m < nLoci; m++)
				stotal += rs[m]; 
			
			if(stotal > m_upper_switches) 
			{
				for (int m = 1; m < nLoci; m++)
				{
					double tr = rs[m];  
					double tt = 1-exp(log(1-tr) / stotal * m_upper_switches);
					(pMP+e)->Setrs(m, tt); 
				}
			}

			///////////////////////////////////////////////
			double likeinc = totalloglike - lastlike; 
			lastlike = totalloglike; 

			char buf[100]; 
			sprintf(buf, "%8.3f\t%8.3f\t%8.3f\t%8.3f\t%8.3f", totalloglike, likeinc, ktotal, stotal, utotal);    
			if(comp_mode == 1)  cout << " ### Linear approximation ";   
			else cout << " ### Quadratic "; 
			cout << e << " " << steps << "\t " << buf << endl; 

			if(comp_mode == 1)  fplog << " ### Linear approximation ";   
			else fplog << " ### Quadratic "; 
			fplog << e << " " << steps << "\t " << buf << endl; 
		}
	}

	Free2DMatrix(sum_top); 
	Free2DMatrix(sum_bot); 
	Free2DMatrix(sum_jmsk); 
	Free2DMatrix(sum_jms); 
	Free2DMatrix(sum_lod); 
	Free2DMatrix(sum_lps); 
	delete[] sum_ujs; 
}                   


void ModelnData::compute_ps(int out_mode, int comp_mode)
{
	int unit = 1000; 
	int nfile = ceil(nLoci / (double) unit); 
	vector<int> pbreak; 
	for (int i = 0; i < nfile; i++)
		pbreak.push_back(i * unit); 
	pbreak.push_back(nLoci); 
//	cout << nfile << " --- "; 
//	for (unsigned i = 0; i < pbreak.size(); i++)
//		cout << pbreak.at(i) << " " ; 
//	cout << endl; 

	fstream outfile; 
	string fn("output/");
	fn.append(fnOutput);
	char buf[100];
	sprintf(buf, ".ps%d%d.txt", comp_mode, out_mode); 
	fn.append(buf);

	outfile.open(fn.c_str(), ios::out);
	if(!outfile.good()) 
	{
		fplog << "ERROR: failed to open file to write." << endl;
		cout << "-warning: failed to open file to write" << endl;
		return; 
	}


	if(unit > nLoci) 
	{
		for (int i = 0; i < nIndiv; i++)
		{
			if(pIndiv[i]->GetisPanel() || pIndiv[i]->get_mask() == 1) continue; 
			pIndiv[i]->compute_ps(nS, nK, pMP, comp_mode); 
			double ** ps = pIndiv[i]->get_ps(); 

			char buf[100]; 
			for (int m = 0; m < nLoci; m++)
			{
				for (int s = 0; s < nS; s++)
				{
					sprintf(buf, "%5.3f ", ps[m][s]); 
					outfile << buf; 
				}
				outfile << " "; 
			}
			outfile << endl; 

			pIndiv[i]->clean_after_em(1); 
		}
	}
	else 
	{
		for (int i = 0; i < nIndiv; i++)
		{
			if(pIndiv[i]->GetisPanel() || pIndiv[i]->get_mask() == 1) continue; 
			for (int b = 0; b < nfile; b++)
			{
				pIndiv[i]->compute_ps(nS, nK, pMP, comp_mode, b, pbreak); 
				double ** ps = pIndiv[i]->get_ps(); 

				char buf[100]; 
				for (int p = 0; p < pbreak.at(b+1)-pbreak.at(b); p++)
				{
					for (int s = 0; s < nS; s++)
					{
						sprintf(buf, "%5.3f ", ps[p][s]); 
						outfile << buf; 
					}
					outfile << " "; 
				}
			}
			outfile << endl; 

			pIndiv[i]->clean_after_em(1); 
		}

	}
//	else //out_mode == 2; //quadratic  
//	{
//		for (int i = 0; i < nIndiv; i++)
//		{
//			if(pIndiv[i]->GetisPanel()) continue; 
//			pIndiv[i]->compute_ps(nS, nK, pMP, 2); 
//			double ** ps = pIndiv[i]->get_ps(); 
//
//			char buf[100]; 
//			for (int m = 0; m < nLoci; m++)
//			{
//				for (int s = 0; s < nS; s++)
//					for (int s1 = 0; s1 <= s; s1++)
//					{
//						sprintf(buf, "%5.3f ", ps[m][nS*s+s1]); 
//						outfile << buf; 
//					}
//				outfile << " "; 
//			}
//			outfile << endl; 
//
//			pIndiv[i]->clean_after_em(1); 
//		}
//
//	}
	//* the above block output the ps(s1,s2). if this were used, the ps in diploid has to be two dimension. 
	outfile.close(); 
}

void ModelnData::compute_pk(int out_mode, int comp_mode)
{
	fstream outfile; 
	string fn("output/");
	fn.append(fnOutput);
	char buf[100];
	sprintf(buf, ".pk%d.txt", out_mode); 
	fn.append(buf);

	outfile.open(fn.c_str(), ios::out);
	if(!outfile.good()) 
	{
		fplog << "ERROR: failed to open file to write." << endl;
		cout << "-warning: failed to open file to write" << endl;
		return; 
	}

	if(comp_mode == 1) 
	{
		for (int e = 0; e < nEMRuns; e++)
		{
			for (int i = 0; i < nIndiv; i++)
			{
//				cout << i << " --- " << pIndiv[i]->GetisPanel() << " "; 
				if(pIndiv[i]->GetisPanel()) continue; 
				pIndiv[i]->compute_pk(nS, nK, pMP+e, 1); 
				double ** pk = pIndiv[i]->get_pk(); 
				if(pk == NULL) continue; 

				char buf[100]; 
				for (int m = 0; m < nLoci; m++)
				{
					for (int k = 0; k < nK; k++)
					{
						sprintf(buf, "%5.3f ", pk[m][k]); 
						outfile << buf; 
					}
					outfile << " "; 
				}
				outfile << endl; 

				pIndiv[i]->clean_after_em(1); 
//				cout << i << endl; 
			}
			outfile << endl; 
		}
	}
	else //compute mode == 2 && diploid; 
	{
		for (int i = 0; i < nIndiv; i++)
		{
			if(pIndiv[i]->GetisPanel()) continue; 
			pIndiv[i]->compute_pk(nS, nK, pMP, 2); 
			double ** pk = pIndiv[i]->get_pk(); 

			char buf[100]; 
			for (int m = 0; m < nLoci; m++)
			{
				for (int k = 0; k < nK; k++)
				{
					double mk = pk[m][k]; 
					sprintf(buf, "%5.3f ", mk); 
					outfile << buf; 
				}
				outfile << " "; 
			}
			outfile << endl; 

			pIndiv[i]->clean_after_em(1); 
		}
	}

	outfile.close(); 
}

void ModelnData::compute_mgt(int comp_mode)
{
	for (int i = 0; i < nIndiv; i++)
	{
		if(pIndiv[i]->GetisPanel() || pIndiv[i]->get_mask() == 1) continue; 
		for (int e = 0; e < nEMRuns; e++)
		{
			pIndiv[i]->compute_mgt(nS, nK, pMP+e, comp_mode, e); 
			pIndiv[i]->clean_after_em(0); 
		}
	}
}

void ModelnData::compute_snp_dstr(void)
{
	for (int i = 0; i < nIndiv; i++)
	{
		if(pIndiv[i]->GetisPanel() || pIndiv[i]->get_mask() == 1) continue; 
		for (int e = 0; e < nEMRuns; e++)
		{
			pIndiv[i]->compute_snp_dstr(nS, nK, pMP+e, e); 
			pIndiv[i]->clean_after_em(0); 
		}
	}
}


void ModelnData::play_ground(void)
{
	cout << "in play ground" << endl; 
 

	//play_ground -num 1000 -C 1 -c 1; 
	int ni = (int) m_num; 
	int ns = nS; 
//	int nc = nK; 
	double ** mgt = Allocate2DMatrix(ni, ns); 
	double * ph = new double[ni]; 

	string fn; 
	fn.assign(fnFILE); 
	ifstream infile; 
	streambuf * pbuf;
	char delimit[] = ";, \t";
	infile.open(fn.c_str(), ios::in);
	if(!infile.is_open()) 
	{
		cout << "-warning: cannot open ps1 file: " << endl; 
		cout << fn << endl; 
		exit(0); 
	} 
	pbuf = infile.rdbuf();
	// open file; 

	string line; 
	line.assign(getline(pbuf)); 
		
	int ii = 0; 
	while(!line.empty())
	{   
		char *res = strtok((char*)line.c_str(), delimit); 
		if(res == NULL) break; 
		ph[ii] = atof(res); 
		int jj = 0; 
		while(1)
		{          
			string tmp;
			res = strtok(NULL, delimit); 
			if(res == NULL) break; 
			mgt[ii][jj] = atof(res); 
			jj++; 
		}   
		line.assign(getline(pbuf));
		ii++; 
	}

    double logbf = bf_linear(ni, ns, ph, mgt, NULL, 0, NULL);  
	cout << logbf << endl; 

//		fstream outfile; 
//		fn.assign(fnDOC); 
//		outfile.open(fn.c_str(), ios::out);
//		if(!outfile.good()) 
//		{
//			cout << "-warning: failed to open file to write" << endl;
//			exit(0);  
//		}
//		outfile.close(); 

	cout << "finishing writing in play ground" << endl; 

}

void ModelnData::simulate_admixed(void)
{
//	{
//	   cout << "in maf-diff block " << endl;  
//		string fn; 
//		fstream outfile; 
//
//		fn.append(fnOutput);
//		fn.append(".mafdiff.txt");
//		outfile.open(fn.c_str(), ios::out);
//		if(!outfile.good()) 
//		{
//			fplog << "ERROR: failed to open file to write." << endl;
//			cout << "-warning: failed to open file to write" << endl;
//			return; 
//		}
//
//		outfile << "###maf-diff.txt" << endl; 
//		for (int m = 0; m < nLoci; m++)
//		{
//			double af1 = 0; 
//			for (int i = 0; i < 90; i++)
//			{
//				short gt = pIndiv[i]->GetsnpGT(m);
//				af1 += gt; 
//			}
//			af1 /= 90.0; 
//
//			double af2 = 0; 
//			for (int i = 91; i < 180; i++)
//			{
//				short gt = pIndiv[i]->GetsnpGT(m);
//				af2 += gt; 
//			}
//			af2 /= 90.0; 
//
//			if(fabs(af1 - af2) > 0.001) 
//			{
//				outfile << af1 << " \t" << af2 << endl; 
////				cout << af1 << " \t" << af2 << endl; 
//			}
//		}
//		outfile.close(); 
//	}
//	return; 

	short ** snpgt = Allocate2DShortMatrix(nIndiv, nLoci); 
    for (int i = 0; i < nIndiv; i++)
	{
		for (int m = 0; m < nLoci; m++)
		{
			short gt = pIndiv[i]->GetsnpGT(m); 
			snpgt[i][m] = gt; 
		}
	}

	int nsnp = nMaxSteps;  
	cout << "nsnp = " << nsnp << endl; 
	cout << "nloci = " << nLoci << endl; 

	short ** c1 = Allocate2DShortMatrix(60, nLoci); 
	short ** c2 = Allocate2DShortMatrix(60, nLoci); 

	for (int k = 0; k < 10; k++)
	{
		short * pf = &snpgt[120-k*2-1][0]; 
		short * pm = &snpgt[240-k*2-1][0]; 
		short * pc = &snpgt[420-k*2-1][0]; 
		int cur = 0; 
		int rand = 0; 
		while (cur < nLoci)
		{
//			int len = gsl_ran_exponential(gsl_r, (double)nsnp); 
			int len = nsnp; 
//			double rand =  gsl_rng_uniform(gsl_r);
			rand++; 
//			if( rand < 0.47) 
			if(rand % 3 == 0) 
			{
				for (int m = cur; m < min(nLoci, cur+len); m++)
				{
					c1[k][m] = pf[m]; 
					fplog << " 0";
				}
			}
//			else if(rand < 0.95) 
			else if (rand % 3 == 1) 
			{
				for (int m = cur; m < min(nLoci, cur+len); m++)
				{
					c1[k][m] = pm[m]; 
					fplog << " 1"; 
				}
			}
			else
			{
				for (int m = cur; m < min(nLoci, cur+len); m++)
				{
					c1[k][m] = pc[m]; 
					fplog << " 2"; 
				}

			}
			cur += len; 
		}
		fplog << endl; 

		pf = &snpgt[120-k*2-2][0]; 
		pm = &snpgt[240-k*2-2][0]; 
		pc = &snpgt[420-k*2-2][0]; 
		cur = 0; 
//		int rand = 1; 
		while (cur < nLoci)
		{
			int len = gsl_ran_exponential(gsl_r, (double)nsnp); 
//			double rand =  gsl_rng_uniform(gsl_r);
			rand++; 
//			if( rand < 0.6) 
			if(rand % 3 == 0) 
			{
				for (int m = cur; m < min(nLoci, cur+len); m++)
				{
					c2[k][m] = pf[m]; 
					fplog << " 0";
				}
			}
//			else if(rand < 0.8) 
			if(rand % 3 == 1) 
			{
				for (int m = cur; m < min(nLoci, cur+len); m++)
				{
					c2[k][m] = pm[m]; 
					fplog << " 1"; 
				}
			}
			else
			{
				for (int m = cur; m < min(nLoci, cur+len); m++)
				{
					c2[k][m] = pc[m]; 
					fplog << " 2"; 
				}

			}
			cur += len; 
		}
		fplog << endl; 
	}

    cout << "i am here " << endl; 

	string fn; 
	fstream outfile; 

	fn.append(fnOutput);
	fn.append(".admixed.txt");
	outfile.open(fn.c_str(), ios::out);
	if(!outfile.good()) 
	{
		fplog << "ERROR: failed to open file to write." << endl;
		cout << "-warning: failed to open file to write" << endl;
		return; 
	}
	
	outfile << nDip << endl; 
	outfile << nLoci << endl; 
	for (int m = 0; m < nLoci; m++)
	{
		string rs(vsRsnum.at(m)); 
		outfile << rs << " "; 
//		for (int i = 0; i < 50; i++)
//		{
//			short gt = pIndiv[2*i]->GetsnpGT(m);
//			gt += pIndiv[2*i+1]->GetsnpGT(m);
//			if(gt == 0)  outfile << " " << mapRs2mm[rs].first << mapRs2mm[rs].first; 
//			else if(gt == 1)  outfile << " " << mapRs2mm[rs].first << mapRs2mm[rs].second; 
//			else  outfile << " " << mapRs2mm[rs].second << mapRs2mm[rs].second; 
//		}
//		for (int i = 0; i < 50; i++)
//		{
//			short gt = pIndiv[120+2*i]->GetsnpGT(m);
//			gt += pIndiv[120+2*i+1]->GetsnpGT(m);
//			if(gt == 0)  outfile << " " << mapRs2mm[rs].first << mapRs2mm[rs].first; 
//			else if(gt == 1)  outfile << " " << mapRs2mm[rs].first << mapRs2mm[rs].second; 
//			else  outfile << " " << mapRs2mm[rs].second << mapRs2mm[rs].second; 
//		}

        for(int k = 0; k < 10; k++)
		{
			short gt = c1[k][m] + c2[k][m]; 
			if(gt == 0)  outfile << " " << mapRs2mm[rs].first << mapRs2mm[rs].first; 
			else if(gt == 1)  outfile << " " << mapRs2mm[rs].first << mapRs2mm[rs].second; 
			else  outfile << " " << mapRs2mm[rs].second << mapRs2mm[rs].second; 
		}
		outfile << endl; 
	}
	outfile.close(); 

	delete[] c1; 
	delete[] c2; 
}

void ModelnData::hpass_pk(int comp_mode)
{
    vector<double> tph;
	std::fstream in;
	in.open(fnFILE.c_str());
	if(in.is_open()) {
		double temp = 0; 
		while (in >> temp) {
		  	tph.push_back(temp);
		}
		in.close();
	}
	else std::cout << "unable to open file" << std::endl;
 
	int ni = 0; 
	for (unsigned i = 0; i < m_phval.size(); i++)
		ni += (int) (m_phval.at(i) == 1);  

	if((int) tph.size() != ni) {
		cout << "incorrect number of entries in " << fnFILE << endl; 
		exit(0); 
	}
	double * ph = new double[ni];
    for (int i = 0; i < ni; i++) 
		ph[i]=tph.at(i);
	vector<double> ().swap(tph); 
	
	double * ph2 = new double[ni];
    for (int i = 0; i < ni; i++) 
		ph2[i] = ph[i]; 

	double ** cov = NULL; 
	int nc = 0; 
	{
		in.open(fnDOC.c_str());
		if(in.is_open() == 1) {
			vector<vector<double> > b;
			std::istringstream iss;
			std::string buf;
			vector<double> tmpvec;
			while(getline(in, buf)){
			   iss.str(buf);
			   double tmp;
			   while(iss>>tmp){
				   tmpvec.push_back(tmp);
			   }
			   b.push_back(tmpvec);
			   tmpvec.clear() ;
			   iss.clear();
			}
			in.close();
			if((int) b.size() == ni) {
				nc = b.at(0).size(); 
				cov = Allocate2DMatrix(ni, nc);  
				for (int i = 0; i < ni; i++)
					for (int j = 0; j < nc; j++)
						cov[i][j] = b.at(i).at(j); 
			}
			else if(b.size() > 0)
			{
				cout << " dimension not match in cov !";
				fplog << " ###### dimension not match in cov !";
				exit(0); 
			}
			vector<vector<double> > ().swap(b); 
		}
		else {
//			cov = Allocate2DMatrix(ni, nc+1);  
		}

	}
	cout << "Number of columns in -DOC " << nc << endl; 
    
//	double ** bf3 = Allocate2DMatrix(nLoci, nEMRuns); 
//	double ** pv3 = Allocate2DMatrix(nLoci, nEMRuns); 
	double ** bf2 = Allocate2DMatrix(nLoci, nEMRuns); 
	double ** pv2 = Allocate2DMatrix(nLoci, nEMRuns); 


	double * beta = new double[2*nK+1+nc]; 
	double ** mgt = Allocate2DMatrix(ni, 2*nK); 
	double * df = new double[nLoci]; 
	double * df2 = new double[nLoci]; 
	for (int m = 0; m < nLoci; m++)
		df[m] = df2[m] = nK; 

	cout << endl << "##num loci = " << nLoci << endl; 
	int unit = 1000; 
//	if(nLoci > unit) 
	if(1) 
	{ 
		int nfile = ceil(nLoci / (double) unit); 
		vector<int> pbreak; 
		for (int i = 0; i < nfile; i++)
			pbreak.push_back(i * unit); 
		pbreak.push_back(nLoci); 
		cout << nfile << " --- "; 
		for (unsigned i = 0; i < pbreak.size(); i++)
			cout << pbreak.at(i) << " " ; 
		cout << endl; 
		//bpreak has 11 elements; 

		double ** mpk = Allocate2DMatrix(ni, unit*nK); 
		for (int e = 0; e < nEMRuns; e++)
		{
//			double ** theta = (pMP+e)->Gettheta(); 
			for (int j = 0; j < nfile; j++)
			{
				cout << j << " of " << nfile << " many blocks" << endl; 
				int ii = 0; 
				for (int i = 0; i < nIndiv; i++)
				{
					if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
					pIndiv[i]->compute_pk(nS, nK, (pMP+e), comp_mode, j, pbreak); 
					double ** pk = pIndiv[i]->get_pk(); 
   					for (int s = 0; s < pbreak[j+1] - pbreak[j]; s++)
						for (int k = 0; k < nK; k++)
							mpk[ii][s*nK+k] = pk[s][k]; 
					ii++;
					pIndiv[i]->clean_after_em(1); 
				}

				for (int s = 0; s < pbreak[j+1] - pbreak[j]; s++)
				{
					int m = s + pbreak[j]; 
					int ii = 0;
//					for (int i = 0; i < nIndiv; i++)
//					{
//						if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
////						double temp = pIndiv[i]->GetsnpGT(m);  
////						if(temp > 3) ph[ii] = 123456; 
////						else 
//							ph[ii] = ph2[ii]; 
//					
////						double impute = 0; 
////						for (int k = 0; k < nK; k++)
////							impute += mpk[ii][s*nK+k] * theta[m][k]; 
////						if(temp > 3) mgt[ii][0] = impute; 
////						else mgt[ii][0] = temp; 
////						mgt[ii][0] = temp; 
////						mgt[ii][1] = impute; 
//						for (int k = 0; k < nK; k++)
//							mgt[ii][k]= mpk[ii][s*nK+k] * theta[m][k]; 
//						ii++;
//					}
//					
//					bf3[m][e] = bf_linear(ni, nK, ph, mgt, beta, nc, cov);  
////					pv3[m][e] = lrt_logistic_QR(ni, nK, ph, mgt, beta, nc, cov,df[m]);  
//					///////////////////////////////////////////////////////////
//
//					ii = 0;
					for (int i = 0; i < nIndiv; i++)
					{
						if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
//						double temp = pIndiv[i]->GetsnpGT(m);  
//						if(temp > 3) ph[ii] = 123456; 
//						else 
							ph[ii] = ph2[ii]; 
					
						for (int k = 0; k < nK; k++)
							mgt[ii][k] = mpk[ii][s*nK+k]; 
						ii++;
					}
					
				    bf2[m][e] = bf_linear(ni, nK, ph, mgt, beta, nc, cov);  
				    pv2[m][e] = lrt_logistic(ni, nK, ph, mgt, beta, nc, cov);  
				}
			}
		}
		Free2DMatrix(mpk); 
		vector<int> ().swap(pbreak); 
	}
//	else 
//	{
//		double *** pk3 = Allocate3DMatrix(nEMRuns, ni*nLoci, nK); 
//		for (int e = 0; e < nEMRuns; e++)
//		{
//			int ii = 0; 
//			for (int i = 0; i < nIndiv; i++)
//			{
//				if(pIndiv[i]->get_mask() == 1) continue; 
//				pIndiv[i]->compute_pk(nS, nK, (pMP+e), comp_mode); 
//				double ** pk = pIndiv[i]->get_pk(); 
//				for (int m = 0; m < nLoci; m++)
//					for (int k = 0; k < nK; k++)
//						pk3[e][ii*nLoci+m][k] = pk[m][k]; 
//				
//				pIndiv[i]->clean_after_em(0); 
//				ii++; 
//			}
//		}
//
//		int ns = nK; 
//		gsl_matrix * Q1 = gsl_matrix_alloc(ni, ns); 
//		gsl_vector * S1 = gsl_vector_alloc(ns); 
//		gsl_matrix * Q2 = gsl_matrix_alloc(ni, ns); 
//		gsl_vector * S2 = gsl_vector_alloc(ns); 
//		
//		gsl_matrix * R = gsl_matrix_alloc(ns, ns); //; 
//		gsl_matrix * QtQ = gsl_matrix_alloc(ns, ns); //; 
//		gsl_matrix * QtQ1 = gsl_matrix_alloc(ns, ns); //; 
//		gsl_matrix * QtQ2 = gsl_matrix_alloc(ns, ns); //; 
//
//		gsl_matrix * T = gsl_matrix_alloc(ns, ns); //; 
//		gsl_matrix * V = gsl_matrix_alloc(ns, ns); //T=R R^t; 
//		gsl_vector * work = gsl_vector_alloc(ns); 
//
//		for (int m = 0; m < nLoci; m++)
//		{
//			//grant;
//
//			gsl_matrix_set_zero(R); 
//			for (int i = 0; i < ni; i++)
//				for (int k = 0; k < nK; k++)
//					gsl_matrix_set(Q1, i, k, pk3[0][i*nLoci+m][k]); 
//			gsl_linalg_SV_decomp_mod(Q1, T, V, S1, work);    
//			for (int s = 0; s < ns; s++)
//			{
//				double temp = gsl_vector_get(S1, s); 
//				gsl_matrix_set(R, s, s, temp * temp); 
//			}
//			for (int e = 1; e < nEMRuns; e++)
//			{
//				for (int i = 0; i < ni; i++)
//					for (int k = 0; k < nK; k++)
//						gsl_matrix_set(Q2, i, k, pk3[e][i*nLoci+m][k]); 
//				gsl_linalg_SV_decomp_mod(Q2, T, V, S2, work);    
//				gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1.0, Q1, Q2, 0.0, QtQ); 
//				gsl_matrix_memcpy(QtQ1, QtQ); 
//				gsl_matrix_memcpy(QtQ2, QtQ); 
//				for (int s = 0; s < ns; s++)
//				{
//					gsl_vector_view tv = gsl_matrix_column(QtQ1, s); 
//					double temp = gsl_vector_get(S2, s); 
//					gsl_vector_scale(&tv.vector, temp*temp); 
//				}
//				gsl_blas_dgemm(CblasNoTrans, CblasTrans, 1.0, QtQ1, QtQ2, 0.0, QtQ); 
//				gsl_matrix_add(R, QtQ); 
//			}
//			gsl_matrix_scale(R, 1.0/nEMRuns); 
//			gsl_linalg_SV_decomp_mod(R, T, V, S1, work);    
//			for (int s = 0; s < ns; s++)
//			{
//				gsl_vector_view tv = gsl_matrix_column(R, s); 
//				double temp = sqrt(gsl_vector_get(S1, s)); 
//				gsl_vector_scale(&tv.vector, temp); 
//			}
//
//			gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, Q1, R, 0.0, Q2); 
//			for (int i = 0; i < ni; i++)
//				for (int k = 0; k < nK; k++)
//					mgt[i][k] = gsl_matrix_get(Q2, i, k); 
//
//			int ii = 0; 
//			for (int i = 0; i < nIndiv; i++)
//			{
//				if(pIndiv[i]->get_mask() == 1) continue; 
//				double gt = pIndiv[i]->GetsnpGT(m); 
//				if(gt > 3) ph[ii] = 123456; 
//				else ph[ii] = ph2[ii]; 
//				ii++; 
//			}
//
//			bf2[m][0] = bf_linear_QR(ni, nK, ph, mgt, beta, nc, cov);  
//			pv2[m][0] = lrt_logistic_QR(ni, nK, ph, mgt, beta, nc, cov, df2[m]);  
//
//		}
//
//		gsl_matrix_free(T); 
//		gsl_matrix_free(V); 
//		gsl_vector_free(work); 
//
//		gsl_vector_free(S1); 
//		gsl_vector_free(S2); 
//		gsl_matrix_free(Q1); 
//		gsl_matrix_free(Q2); 
//
//		gsl_matrix_free(R); 
//		gsl_matrix_free(QtQ); 
//		gsl_matrix_free(QtQ1); 
//		gsl_matrix_free(QtQ2); 
//
//		Free3DMatrix(pk3); 
//	}

    double * bf1 = new double[nLoci];
	double * pv1 = new double[nLoci];
	{
		for (int m = 0; m < nLoci; m++)
		{
			int ii = 0; 
			for (int i = 0; i < nIndiv; i++)
			{
				if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
				mgt[ii][0] = pIndiv[i]->GetsnpGT(m); 
				if(mgt[ii][0] > 3) ph[ii] = 123456; 
				else ph[ii] = ph2[ii]; 
			   	ii++; 
			}

		   bf1[m] = bf_linear(ni, 1, ph, mgt, beta, nc, cov);  
		   pv1[m] = lrt_logistic(ni, 1, ph, mgt, beta, nc, cov);  
		}
	}

	fstream fo; 
	string fn("output/");
	fn.append(fnOutput);
	fn.append(".bf.txt");

	fo.open(fn.c_str(), ios::out);
	if(!fo.good()) 
	{
		fplog << "ERROR: failed to open file to write." << endl;
		cout << "-warning: failed to open file to write" << endl;
		return; 
	}

//	fo << " bf3\t pv3\t bf2\t pv2\t bf1\t pv1" << endl; 
//	fo << " bfx\t bf2\t bf1\t pv1" << endl; 
	fo << " bf2\t pv2\t bf1\t pv1" << endl; 
	for (int m = 0; m < nLoci; m++)
	{
		char buf[100]; 
		double b1,b2,b3;
		b1=b2=b3=0;
//		b3 = (sumlog(bf3[m], nEMRuns) - log((double)nEMRuns)) / log(10.0);
		b2 = (sumlog(bf2[m], nEMRuns) - log((double)nEMRuns)) / log(10.0);
		b1 =bf1[m] / log(10.0); 
//		sprintf(buf, " %5.3f \t %5.3f \t %5.3f\n", b3, b2, b1); 
//		fo << buf; 
                                                       
//		double p1, p2, p3;
//		p1=p2=p3=0; 
//		for (int e = 0; e < nEMRuns; e++)
//			if(p3 < pv3[m][e]) p3 = pv3[m][e]; 
//		p3 = -log10(gsl_cdf_chisq_Q(p3, nK));
//		if(p3 > 400) p3 = 400; 
//
		double p2 = 0; 
		for (int e = 0; e < nEMRuns; e++)
			if(p2 < pv2[m][e]) p2 = pv2[m][e]; 
		p2 = -log10(gsl_cdf_chisq_Q(p2, (int)df2[m]));
		if(p2 > 400) p2 = 400; 

		double p1 = -log10(gsl_cdf_chisq_Q(pv1[m], 1));
		sprintf(buf, " %5.3f \t %5.3f \t %5.3f \t %5.3f\n", b2, p2, b1, p1); 
		fo << buf; 
	}
	fo.close(); 

	Free2DMatrix(mgt); 
	Free2DMatrix(cov); 
	delete[] ph; 
	delete[] ph2; 
	delete[] beta; 
//	Free2DMatrix(bf3); 
	Free2DMatrix(bf2); 
//	Free2DMatrix(pv3); 
	Free2DMatrix(pv2); 
	delete[] bf1; 
	delete[] pv1; 
	delete[] df; 
	delete[] df2; 
}

void ModelnData::hpass_admix(int comp_mode)
{
    vector<double> tph;
	std::fstream in;
	in.open(fnFILE.c_str());
	if(in.is_open()) {
		double temp = 0; 
		while (in >> temp) {
		  	tph.push_back(temp);
		}
		in.close();
	}
	else std::cout << "unable to open file" << std::endl;
 
	int ni = 0; 
	for (unsigned i = 0; i < m_phval.size(); i++)
		ni += (int) (m_phval.at(i) == 1);  

	if((int) tph.size() != ni) {
		cout << "incorrect number of entries in " << fnFILE << endl; 
		exit(0); 
	}
	double * ph = new double[ni];
    for (int i = 0; i < ni; i++) 
		ph[i]=tph.at(i);
	vector<double> ().swap(tph); 
	
	double * ph2 = new double[ni];
    for (int i = 0; i < ni; i++) 
		ph2[i] = ph[i]; 

	double ** cov = NULL; 
	int nc = 0; 
	{
		in.open(fnDOC.c_str());
		if(in.is_open() == 1) {
			vector<vector<double> > b;
			std::istringstream iss;
			std::string buf;
			vector<double> tmpvec;
			while(getline(in, buf)){
			   iss.str(buf);
			   double tmp;
			   while(iss>>tmp){
				   tmpvec.push_back(tmp);
			   }
			   b.push_back(tmpvec);
			   tmpvec.clear() ;
			   iss.clear();
			}
			in.close();
			if((int) b.size() == ni) {
				nc = b.at(0).size(); 
				cov = Allocate2DMatrix(ni, nc);  
				for (int i = 0; i < ni; i++)
					for (int j = 0; j < nc; j++)
						cov[i][j] = b.at(i).at(j); 
			}
			else if(b.size() > 0)
			{
				cout << " dimension not match in cov !";
				fplog << " ###### dimension not match in cov !";
				exit(0); 
			}
			vector<vector<double> > ().swap(b); 
		}
		else {
//			cov = Allocate2DMatrix(ni, nc+1);  
		}

	}
	cout << "Number of columns in -DOC " << nc << endl; 
    
	double ** bf5 = Allocate2DMatrix(nLoci, nEMRuns); 
	double ** bf4 = Allocate2DMatrix(nLoci, nEMRuns); 
	double ** bf3 = Allocate2DMatrix(nLoci, nEMRuns); 
	double ** bf3s1 = Allocate2DMatrix(nLoci, nEMRuns); 
	double ** bf3s2 = Allocate2DMatrix(nLoci, nEMRuns); 
	double ** bf3s3 = Allocate2DMatrix(nLoci, nEMRuns); 
//	double ** pv3 = Allocate2DMatrix(nLoci, nEMRuns); 
	double ** bf2 = Allocate2DMatrix(nLoci, nEMRuns); 
//	double ** pv2 = Allocate2DMatrix(nLoci, nEMRuns); 
	double ** bf1s = Allocate2DMatrix(nLoci, nEMRuns); 
//	double ** pv1s = Allocate2DMatrix(nLoci, nEMRuns); 


	double * beta = new double[2*nK+1+nc]; 
	double ** mgt = Allocate2DMatrix(ni, 2*nK); 
	double ** cov2 = Allocate2DMatrix(ni, nS+nc); 
	double * df = new double[nLoci]; 
	double * df2 = new double[nLoci]; 
	for (int m = 0; m < nLoci; m++)
		df[m] = df2[m] = nK; 

	cout << endl << "##num loci = " << nLoci << endl; 
	int unit = 1000; 
	{ 
		int nfile = ceil(nLoci / (double) unit); 
		vector<int> pbreak; 
		for (int i = 0; i < nfile; i++)
			pbreak.push_back(i * unit); 
		pbreak.push_back(nLoci); 
		cout << nfile << " --- "; 
		for (unsigned i = 0; i < pbreak.size(); i++)
			cout << pbreak.at(i) << " " ; 
		cout << endl; 
		//bpreak has 11 elements; 

		double ** mps = Allocate2DMatrix(ni, unit*nS); 
		double ** mpk = Allocate2DMatrix(ni, unit*nK); 
		for (int e = 0; e < nEMRuns; e++)
		{
			double ** theta = (pMP+e)->Gettheta(); 
			for (int j = 0; j < nfile; j++)
			{
				cout << j << " of " << nfile << " many blocks" << endl; 
				int ii = 0; 
				for (int i = 0; i < nIndiv; i++)
				{
					if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
					pIndiv[i]->compute_pk(nS, nK, (pMP+e), comp_mode, j, pbreak); 
					double ** ps = pIndiv[i]->get_ps(); 
   					for (int s = 0; s < pbreak[j+1] - pbreak[j]; s++)
						for (int t = 0; t < nS; t++)
							mps[ii][s*nS+t] = ps[s][t]; 
					double ** pk = pIndiv[i]->get_pk(); 
   					for (int s = 0; s < pbreak[j+1] - pbreak[j]; s++)
						for (int k = 0; k < nK; k++)
							mpk[ii][s*nK+k] = pk[s][k]; 
					ii++;
					pIndiv[i]->clean_after_em(1); 
				}

				for (int s = 0; s < pbreak[j+1] - pbreak[j]; s++)
				{
					int m = s + pbreak[j]; 

					int ii = 0;
					for (int i = 0; i < nIndiv; i++)
					{
						if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
//						double temp = pIndiv[i]->GetsnpGT(m);  
//						if(temp > 3) ph[ii] = 123456; 
//						else 
							ph[ii] = ph2[ii]; 
					
						for (int k = 0; k < nK; k++)
							mgt[ii][k] = mpk[ii][s*nK+k]; 

						for (int t = 0; t < nc; t++)
							cov2[ii][t] = cov[ii][t]; 
						for (int t = 0; t < nS-1; t++)
							cov2[ii][nc+t] = mps[ii][s*nS+t]; 
						ii++;
					}
				    bf5[m][e] = bf_linear(ni, nK-1, ph, mgt, beta, nc+nS-1, cov2);  
					///////////////////////////////////////////////////////////
					//pk | ps; 

					ii = 0;
					for (int i = 0; i < nIndiv; i++)
					{
						if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
//						double temp = pIndiv[i]->GetsnpGT(m);  
//						if(temp > 3) ph[ii] = 123456; 
//						else 
							ph[ii] = ph2[ii]; 
					
						for (int k = 0; k < nK-1; k++)
							mgt[ii][k] = mpk[ii][s*nK+k]; 
						for (int t = 0; t < nS-1; t++)
							mgt[ii][nK+t] = mps[ii][s*nS+t]; 
						ii++;
					}
				    bf4[m][e] = bf_linear(ni, nK+nS-2, ph, mgt, beta, nc, cov);  
					///////////////////////////////////////////////////////////
					//pk and ps; 

					ii = 0;
					for (int i = 0; i < nIndiv; i++)
					{
						if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
						double temp = pIndiv[i]->GetsnpGT(m);  
//						if(temp > 3) ph[ii] = 123456; 
//						else 
							ph[ii] = ph2[ii]; 

						if(temp > 3) {
							double imp = 0;  
							for (int k = 0; k < nK; k++)
								imp += mpk[ii][s*nK+k] * theta[m][k]; 
							temp = imp; 
						}   //impute the missing genotypes; 
					
						for (int t = 0; t < nS; t++)
							mgt[ii][t]= mps[ii][s*nS+t] * temp; 
						ii++;
					}
					bf3[m][e] = bf_linear(ni, nS, ph, mgt, beta, nc, cov);  
					///////////////////////////////////////////////////////////
					//ps 

					ii = 0;
					for (int i = 0; i < nIndiv; i++)
					{
						if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
//						double temp = pIndiv[i]->GetsnpGT(m);  
//						if(temp > 3) ph[ii] = 123456; 
//						else 
							ph[ii] = ph2[ii]; 
					
						for (int k = 0; k < nK; k++)
							mgt[ii][k] = mpk[ii][s*nK+k]; 
						ii++;
					}
				    bf2[m][e] = bf_linear(ni, nK-1, ph, mgt, beta, nc, cov);  
					///////////////////////////////////////////////////////////
					//pk
					//above three don't involve genotypes; 

					ii = 0;
					for (int i = 0; i < nIndiv; i++)
					{
						if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
						double temp = pIndiv[i]->GetsnpGT(m);  
//						if(temp > 3) ph[ii] = 123456; 
//						else 
							ph[ii] = ph2[ii]; 

						if(temp > 3) {
							double imp = 0;  
							for (int k = 0; k < nK; k++)
								imp += mpk[ii][s*nK+k] * theta[m][k]; 
							temp = imp; 
						}   //impute the missing genotypes; 
					
//						for (int t = 0; t < nS; t++)
							mgt[ii][0]= mps[ii][s*nS+0] * temp; 
						ii++;
					}
					bf3s1[m][e] = bf_linear(ni, 1, ph, mgt, beta, nc, cov);  
					///////////////////////////////////////////////////////////

					ii = 0;
					for (int i = 0; i < nIndiv; i++)
					{
						if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
						double temp = pIndiv[i]->GetsnpGT(m);  
//						if(temp > 3) ph[ii] = 123456; 
//						else 
							ph[ii] = ph2[ii]; 

						if(temp > 3) {
							double imp = 0;  
							for (int k = 0; k < nK; k++)
								imp += mpk[ii][s*nK+k] * theta[m][k]; 
							temp = imp; 
						}   //impute the missing genotypes; 
					
//						for (int t = 0; t < nS; t++)
							mgt[ii][0]= mps[ii][s*nS+1] * temp; 
						ii++;
					}
					bf3s2[m][e] = bf_linear(ni, 1, ph, mgt, beta, nc, cov);  
					///////////////////////////////////////////////////////////

					ii = 0;
					for (int i = 0; i < nIndiv; i++)
					{
						if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
						double temp = pIndiv[i]->GetsnpGT(m);  
//						if(temp > 3) ph[ii] = 123456; 
//						else 
							ph[ii] = ph2[ii]; 

						if(temp > 3) {
							double imp = 0;  
							for (int k = 0; k < nK; k++)
								imp += mpk[ii][s*nK+k] * theta[m][k]; 
							temp = imp; 
						}   //impute the missing genotypes; 
					
//						for (int t = 0; t < nS; t++)
							mgt[ii][0]= mps[ii][s*nS+2] * temp; 
						ii++;
					}
					bf3s3[m][e] = bf_linear(ni, 1, ph, mgt, beta, nc, cov);  
					///////////////////////////////////////////////////////////


					ii = 0;
					for (int i = 0; i < nIndiv; i++)
					{
						if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
						double temp = pIndiv[i]->GetsnpGT(m);  
//						if(temp > 3) ph[ii] = 123456; 
//						else 
							ph[ii] = ph2[ii]; 

						if(temp > 3) {
							double imp = 0;  
							for (int k = 0; k < nK; k++)
								imp += mpk[ii][s*nK+k] * theta[m][k]; 
							temp = imp; 
						}   //impute the missing genotypes; 
					
						mgt[ii][0] = temp; 
						ii++;
					}
				    bf1s[m][e] = bf_linear(ni, 1, ph, mgt, beta, nc, cov);  
//				    pv1s[m][e] = lrt_logistic(ni, 1, ph, mgt, beta, nc, cov);  
				}
			}
		}
		Free2DMatrix(mpk); 
		vector<int> ().swap(pbreak); 
	}

    double * bf1 = new double[nLoci];
	double * pv1 = new double[nLoci];
	{
		for (int m = 0; m < nLoci; m++)
		{
			int ii = 0; 
			for (int i = 0; i < nIndiv; i++)
			{
				if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
				mgt[ii][0] = pIndiv[i]->GetsnpGT(m); 
				if(mgt[ii][0] > 3) ph[ii] = 123456; 
				else ph[ii] = ph2[ii]; 

//				double temp = pIndiv[i]->GetsnpGT(m);  
//				if(temp > 3) {
//					double imp = 0;  
//					for (int k = 0; k < nK; k++)
//						imp += mpk[ii][s*nK+k] * theta[m][k]; 
//					temp = imp; 
//				}   //impute the missing genotypes; 
//				ph[ii] = ph2[ii]; 
//				mgt[ii][0] = temp; 

			   	ii++; 
			}

		   bf1[m] = bf_linear(ni, 1, ph, mgt, beta, nc, cov);  
		   pv1[m] = lrt_logistic(ni, 1, ph, mgt, beta, nc, cov);  
		}
	}

	fstream fo; 
	string fn("output/");
	fn.append(fnOutput);
	fn.append(".bf.txt");

	fo.open(fn.c_str(), ios::out);
	if(!fo.good()) 
	{
		fplog << "ERROR: failed to open file to write." << endl;
		cout << "-warning: failed to open file to write" << endl;
		return; 
	}

//	fo << " bf3\t pv3\t bf2\t pv2\t bf1\t pv1" << endl; 
	fo << " bfc\t bfsk\t bfs \t bfs1 \t bfs2 \t bfs3 \t bfk\t bf1s \t bf1\t pv1 " << endl; 
	for (int m = 0; m < nLoci; m++)
	{
		char buf[100]; 
		double b1,b2,b3,b4,b5;
		b1=b2=b3=b4=b5=0;
		double b3s1, b3s2, b3s3; 
		double b1s = 0; 
		b5 = (sumlog(bf5[m], nEMRuns) - log((double)nEMRuns)) / log(10.0);
		b4 = (sumlog(bf4[m], nEMRuns) - log((double)nEMRuns)) / log(10.0);
		b3 = (sumlog(bf3[m], nEMRuns) - log((double)nEMRuns)) / log(10.0);
		b3s1 = (sumlog(bf3s1[m], nEMRuns) - log((double)nEMRuns)) / log(10.0);
		b3s2 = (sumlog(bf3s2[m], nEMRuns) - log((double)nEMRuns)) / log(10.0);
		b3s3 = (sumlog(bf3s3[m], nEMRuns) - log((double)nEMRuns)) / log(10.0);
		b2 = (sumlog(bf2[m], nEMRuns) - log((double)nEMRuns)) / log(10.0);
		b1s = (sumlog(bf1s[m], nEMRuns) - log((double)nEMRuns)) / log(10.0);
		b1 =bf1[m] / log(10.0); 
//		double b1s  = (sumlog(bf1s[m], nEMRuns) - log((double)nEMRuns)) / log(10.0);
//		sprintf(buf, " %5.3f \t %5.3f \t %5.3f\n", b3, b2, b1); 
//		fo << buf; 
                                                       
//		double p1, p2, p3;
//		p1=p2=p3=0; 
//		for (int e = 0; e < nEMRuns; e++)
//			if(p3 < pv3[m][e]) p3 = pv3[m][e]; 
//		p3 = -log10(gsl_cdf_chisq_Q(p3, nK));
//		if(p3 > 400) p3 = 400; 
//
//		for (int e = 0; e < nEMRuns; e++)
//			if(p2 < pv2[m][e]) p2 = pv2[m][e]; 
//		p2 = -log10(gsl_cdf_chisq_Q(p2, (int)df2[m]));
//		if(p2 > 400) p2 = 400; 
//
		double p1 = -log10(gsl_cdf_chisq_Q(pv1[m], 1));
//		double p1s = -log10(gsl_cdf_chisq_Q(find_max_value(pv1s[m], nEMRuns), 1));
		sprintf(buf, " %5.3f \t %5.3f \t %5.3f \t %5.3f \t %5.3f \t %5.3f \t %5.3f \t %5.3f \t %5.3f \t %5.3f\n", b5, b4, b3, b3s1, b3s2, b3s3, b2, b1s, b1, p1); 
		fo << buf; 
	}
	fo.close(); 

	Free2DMatrix(mgt); 
	Free2DMatrix(cov); 
	Free2DMatrix(cov2); 
	delete[] ph; 
	delete[] ph2; 
	delete[] beta; 
	Free2DMatrix(bf5); 
	Free2DMatrix(bf4); 
	Free2DMatrix(bf3); 
	Free2DMatrix(bf3s1); 
	Free2DMatrix(bf3s2); 
	Free2DMatrix(bf3s3); 
	Free2DMatrix(bf2); 
	Free2DMatrix(bf1s); 
//	Free2DMatrix(pv1s); 
//	Free2DMatrix(pv3); 
//	Free2DMatrix(pv2); 
	delete[] bf1; 
	delete[] pv1; 
	delete[] df; 
	delete[] df2; 
}

void ModelnData::hpass_matrixnormal(int comp_mode)
{
    vector<vector<double> >tph;
	std::fstream in;
	in.open(fnFILE.c_str());
	if(in.is_open()) {
		std::istringstream iss;
		std::string buf;
		vector<double> tmpvec;
		while(getline(in,buf)){
		   iss.str(buf);
		   double tmp;
		   while(iss>>tmp){
			   tmpvec.push_back(tmp);
		   }
		   tph.push_back(tmpvec);
		   tmpvec.clear() ;
		   iss.clear();
		}
		in.close();
	}
	else std::cout << "unable to open file" << std::endl;

//////////////////////////////////////////////////

	int nd=(int) m_xset.size(); 
	int nc2 = (int) m_cset.size(); 
//	if(nd + nc2 != m_nph) {
//		cout << "wrong index sets ..." << endl; 
//		exit(0); 
//	}

	fplog << " ### which columns are in covariates : "; 
	for(unsigned i = 0; i < m_cset.size(); i++)
		fplog << m_cset.at(i) << " "; 
	fplog << endl; 
	fplog << " ### which columns are in design matrix : "; 
	for(unsigned i = 0; i < m_xset.size(); i++)
		fplog << m_xset.at(i) << " "; 
	fplog << endl; 

	cout << " ### which columns are in covariates : "; 
	for(unsigned i = 0; i < m_cset.size(); i++)
		cout << m_cset.at(i) << " "; 
	cout << endl; 
	cout << " ### which columns are in design matrix : "; 
	for(unsigned i = 0; i < m_xset.size(); i++)
		cout << m_xset.at(i) << " "; 
	cout << endl; 

	int phcol = tph.at(0).size() / m_nph;                      
	cout << "### nph  nd phcol nc2 " << m_nph << " " << nd << " " << phcol << " " << nc2 << endl; 
	fplog << "### nph  nd phcol nc2 " << m_nph << " " << nd << " " << phcol << " " << nc2 << endl; 
 
	int ni = 0; 
	for (unsigned i = 0; i < m_phval.size(); i++)
		ni += (int) (m_phval.at(i) == 1);  

	if((int) tph.size() != ni) {
		cout << "incorrect number of entries in " << ni << " " << fnFILE << endl; 
		exit(0); 
	}

	double ** cov2 = Allocate2DMatrix(ni, nc2*phcol); 
	double ** ph2 = Allocate2DMatrix(ni, nd*phcol); 
	double ** mph = Allocate2DMatrix(ni, nd); 
    for (int i = 0; i < ni; i++) 
	{
		for(int c = 0; c < phcol; c++)
		{
			for (int d = 0; d < nd; d++)
				ph2[i][c*nd+d]=tph.at(i).at(c*m_nph+m_xset.at(d));
			for (int d = 0; d < nc2; d++)
				cov2[i][c*nc2+d]=tph.at(i).at(c*m_nph+m_cset.at(d));
		}

	}
	vector<vector<double> >().swap(tph); 


	double ** cov = NULL; 
	int nc = 0; 
	{
		in.open(fnDOC.c_str());
		if(in.is_open() == 1) {
			vector<vector<double> > b;
			std::istringstream iss;
			std::string buf;
			vector<double> tmpvec;
			while(getline(in, buf)){
			   iss.str(buf);
			   double tmp;
			   while(iss>>tmp){
				   tmpvec.push_back(tmp);
			   }
			   b.push_back(tmpvec);
			   tmpvec.clear() ;
			   iss.clear();
			}
			in.close();
			if((int) b.size() == ni) {
				nc = b.at(0).size(); 
				cov = Allocate2DMatrix(ni, nc+nc2);  
				for (int i = 0; i < ni; i++)
					for (int j = 0; j < nc; j++)
						cov[i][j] = b.at(i).at(j); 
			}
			else if(b.size() > 0)
			{
				cout << " dimension not match in cov !";
				fplog << " ###### dimension not match in cov !";
				exit(0); 
			}
			vector<vector<double> > ().swap(b); 
		}
		else {
//			cov = Allocate2DMatrix(ni, nc+1);  
		}

	}
	cout << "number of columns in -DOC mph" << nc << endl; 
	nc += nc2;
//	for (int i = 0; i < ni; i++)
//	{
//		for (int j = 0; j < nc; j++)
//			cout << cov[i][j] << " "; 
//		cout << endl; 
//	}
//	cout << endl; 

	double *** bf3 = Allocate3DMatrix(nLoci, phcol, nEMRuns); 

	double * beta = new double[2*nK+1+nc]; 
	double ** mgt = Allocate2DMatrix(ni, 2*nK); 

	cout << " num loci = " << nLoci << endl; 
	fplog << "### num loci = " << nLoci << endl; 
	int unit = 1000; 
//	if(nLoci > unit) 
	{ 
		int nfile = ceil(nLoci / (double) unit); 
		vector<int> pbreak; 
		for (int i = 0; i < nfile; i++)
			pbreak.push_back(i * unit); 
		pbreak.push_back(nLoci); 
		cout << nfile << " --- "; 
		for (unsigned i = 0; i < pbreak.size(); i++)
			cout << pbreak.at(i) << " " ; 
		cout << endl; 
		//bpreak has 11 elements; 

		double ** mpk = Allocate2DMatrix(ni, unit*nK); 
		for (int e = 0; e < nEMRuns; e++)
		{
//			double ** th = (pMP+e)->Gettheta(); 
			for (int j = 0; j < nfile; j++)
			{
				cout << j << " of " << nfile << " many blocks" << endl; 
				fplog << e << "-th EM run; "  << j << " of " << nfile << " many blocks" << endl;
				int ii = 0; 
				for (int i = 0; i < nIndiv; i++)
				{
					if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
					pIndiv[i]->compute_pk(nS, nK, (pMP+e), comp_mode, j, pbreak); 
					double ** pk = pIndiv[i]->get_pk(); 
   					for (int s = 0; s < pbreak[j+1] - pbreak[j]; s++)
						for (int k = 0; k < nK; k++)
							mpk[ii][s*nK+k] = pk[s][k]; 
					ii++;
					pIndiv[i]->clean_after_em(1); 
				}

				for (int s = 0; s < pbreak[j+1] - pbreak[j]; s++)
				{
					int m = s + pbreak[j]; 

					ii = 0;
					for (int i = 0; i < nIndiv; i++)
					{
						if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
						for (int k = 0; k < nK; k++)
							mgt[ii][k] = mpk[ii][s*nK+k]; 
						ii++;
					}

					for (int c = 0; c < phcol; c++) 
					{
						for (int i = 0; i < ni; i++) 
						{
							for (int d = 0; d < nd; d++)
								mph[i][d] = ph2[i][c*nd+d];
							for (int d = 0; d < nc2; d++)
								cov[i][nc-1-d] = cov2[i][c*nc2+d];
						}
		
						bf3[m][c][e] = bf_matrix_normal(ni, nK, nd, mph, mgt, beta, nc, cov);  
//	    				bf3[m][c][e] = bf_linear(ni, nK, mph[0], mgt, beta, nc, cov);  
					}
				}


			}

		}
		Free2DMatrix(mpk); 
		vector<int> ().swap(pbreak); 
	}
//	else 
//	{
//		for (int e = 0; e < nEMRuns; e++)
//		{
//			double ** th = (pMP+e)->Gettheta(); 
//			for (int i = 0; i < nIndiv; i++)
//			{
//				if(pIndiv[i]->get_mask() == 1) continue; 
//				pIndiv[i]->compute_pk(nS, nK, (pMP+e), comp_mode); 
//				pIndiv[i]->clean_after_em(0); 
//			}
//			for (int m = 0; m < nLoci; m++)
//			{
//				int ii = 0; 
//				for (int i = 0; i < nIndiv; i++)
//				{
//					if(pIndiv[i]->get_mask() == 1) continue; 
//				
//					double ** pk = pIndiv[i]->get_pk(); 
//
//					for (int k = 0; k < nK; k++)
//						mgt[ii][k]= pk[m][k]; 
//					ii++;
//				}
//				for (int c = 0; c < phcol; c++) 
//				{
//					for (int i = 0; i < ni; i++) 
//					{
//						for (int d = 0; d < nd; d++)
//							mph[i][d] = ph2[i][c*nd+d];
//					}
//	
//					bf3[m][c][e] = bf_matrix_normal(ni, nK, nd, mph, mgt, beta, nc, cov);  
//				}
//			}
//		}
//	}

	//output bf-hap; 
	{
		fstream fo; 
		string fn("output/");
		fn.append(fnOutput);
		fn.append(".bf-hap.txt");

		fo.open(fn.c_str(), ios::out);
		if(!fo.good()) 
		{
			fplog << "ERROR: failed to open file to write." << endl;
			cout << "-warning: failed to open file to write" << endl;
			return; 
		}

		for (int m = 0; m < nLoci; m++)
		{
			for (int c = 0; c < phcol; c++)
			{
				double b2 = (sumlog(bf3[m][c], nEMRuns) - log((double)nEMRuns)) / log(10.0);
				char buf[100]; 
				sprintf(buf, " %5.3f ", b2); 
				fo << buf; 
			}
			fo << endl; 

		}
		fo.close(); 
	}

//	if(nLoci > unit) 
	{ 
		int nfile = ceil(nLoci / (double) unit); 
		vector<int> pbreak; 
		for (int i = 0; i < nfile; i++)
			pbreak.push_back(i * unit); 
		pbreak.push_back(nLoci); 
		cout << nfile << " --- "; 
		for (unsigned i = 0; i < pbreak.size(); i++)
			cout << pbreak.at(i) << " " ; 
		cout << endl; 
		//bpreak has 11 elements; 

		double ** mpk = Allocate2DMatrix(ni, unit*nK); 
		for (int e = 0; e < nEMRuns; e++)
		{
			double ** th = (pMP+e)->Gettheta(); 
			for (int j = 0; j < nfile; j++)
			{
				cout << j << " of " << nfile << " many blocks" << endl; 
				fplog << e << "-th EM run; "  << j << " of " << nfile << " many blocks" << endl;
				int ii = 0; 
				for (int i = 0; i < nIndiv; i++)
				{
					if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
					pIndiv[i]->compute_pk(nS, nK, (pMP+e), comp_mode, j, pbreak); 
					double ** pk = pIndiv[i]->get_pk(); 
   					for (int s = 0; s < pbreak[j+1] - pbreak[j]; s++)
						for (int k = 0; k < nK; k++)
							mpk[ii][s*nK+k] = pk[s][k]; 
					ii++;
					pIndiv[i]->clean_after_em(1); 
				}

				for (int s = 0; s < pbreak[j+1] - pbreak[j]; s++)
				{
					int m = s + pbreak[j]; 

					ii = 0;
					for (int i = 0; i < nIndiv; i++)
					{
						if(pIndiv[i]->get_mask() == 1 || pIndiv[i]->GetisPanel() == 1) continue; 
						double temp = pIndiv[i]->GetsnpGT(m);  
						if(temp > 3) {
							mgt[ii][0] = 0; 
							for (int k = 0; k < nK; k++)
								mgt[ii][0] += mpk[ii][s*nK+k] * th[m][k]; 
						}
						else mgt[ii][0] = temp; 
						ii++;
						//impute the missing genotypes; 
					}

					for (int c = 0; c < phcol; c++) 
					{
						for (int i = 0; i < ni; i++) 
						{
							for (int d = 0; d < nd; d++)
								mph[i][d] = ph2[i][c*nd+d];
							for (int d = 0; d < nc2; d++)
								cov[i][nc-1-d] = cov2[i][c*nc2+d];
						}
		
						bf3[m][c][e] = bf_matrix_normal(ni, 1, nd, mph, mgt, beta, nc, cov);  
//	    				bf3[m][c][e] = bf_linear(ni, nK, mph[0], mgt, beta, nc, cov);  
					}
				}
			}

		}
		Free2DMatrix(mpk); 
		vector<int> ().swap(pbreak); 
	}

	//output bf-snp; 
	{
		fstream fo; 
		string fn("output/");
		fn.append(fnOutput);
		fn.append(".bf-snp.txt");

		fo.open(fn.c_str(), ios::out);
		if(!fo.good()) 
		{
			fplog << "ERROR: failed to open file to write." << endl;
			cout << "-warning: failed to open file to write" << endl;
			return; 
		}

		for (int m = 0; m < nLoci; m++)
		{
			for (int c = 0; c < phcol; c++)
			{
				double b2 = (sumlog(bf3[m][c], nEMRuns) - log((double)nEMRuns)) / log(10.0);
				char buf[100]; 
				sprintf(buf, " %5.3f ", b2); 
				fo << buf; 
			}
			fo << endl; 

		}
		fo.close(); 
	}

	Free2DMatrix(mgt); 
	Free2DMatrix(ph2); 
	Free2DMatrix(cov); 
	delete[] beta; 
	Free3DMatrix(bf3); 
}

 
void ModelnData::open_log(void)
{
    string sfn("output/");
    sfn.append(fnOutput);
    sfn.append(".log.txt");
    fplog.open(sfn.c_str(), ios::out);
    if(!fplog.is_open()) 
    {
        cout << "-warning: cannot open log file" << endl;
        safe_exit(); 
    }
}

void ModelnData::close_log(void)
{
    fplog.close();
}

